/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1841 $
 * $Date: 2006-04-14 16:27:07 -0700 (Fri, 14 Apr 2006) $
 * $Author: hedstrom $
 * $Id: b6stream.cpp 1841 2006-04-14 23:27:07Z hedstrom $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

#include "b6stream.hpp"
#include "record_types.hpp"
#include "endl_formats.hpp"
#include "convert.hpp"
#include "global_params.hpp"
#include "messaging.hpp"

extern GlobalParameterClass Global;

// *********** class iMF_array_link ******************
// ----------- iMF_array_link::print_MF ---------------
void iMF_array_link::print_MF( )
{
  cout << "    MT: " << MT( );
  for( iMF_array_link::iterator this_mf = begin( ); 
       this_mf != end( ); ++this_mf )
  {
    cout << "  " << *this_mf;
  }
  cout << endl;
}
// ----------- iMF_array_link::angle_cm ---------------
bool iMF_array_link::angle_cm( )
// function to determine whether angular data should be CM or lab frame
{
  bool CM = true;

  for( iMF_array_link::iterator this_mf = begin( ); 
       this_mf != end( ); ++this_mf )
  {
    // are there energy distributions?
    if( *this_mf == 5 )
    {
      CM = false;
      break;
    }
  }
  return CM;
}

// *********** class iMF_array_list ******************
// ----------- iMF_array_list::at ---------------
// Function to get the link at "x"
// Returns "false" if the entry is not found.
bool iMF_array_list::at(int x, iMF_array_list::iterator& link)
{
  for( link = begin( ); 
       link != end( );
       ++link )
  {
    if ( link->MT( ) == x ) 
    {
      return true;
    }
  }
  return false;
}
// -------------------- iMF_array_list::found ----------------
bool iMF_array_list::found( int mt )
{
  bool found_it = false;

  for( iMF_array_list::iterator this_mt = begin();
       this_mt != end(); ++this_mt)
  {
    if( mt == this_mt->MT( ) )
    {
      found_it = true;
      break;
    }
  }
  return found_it;
}

// *********** class b6stream ******************
// -------------------- b6stream constructor ----------------
b6stream::b6stream(string inputFile)
{
  //zero out all the identifiers
  NTAPE=0;MAT=0;MF=0;MT=0;NS=-1;LRP=0;LFI=0;NLIB=0;
  NMOD=0;LIS=0;LISO=0;NFOR=0;NSUB=0;NVER=0;LDRV=0;NWD=0;
  NXC=0;ZA=0;AWR=0;ELIS=0;STA=0;AWI=0;TEMP=0;
  prev_mt = 0;
  prev_mf = 0;
  //zero out logical flags
  SEND=false;FEND=false;MEND=false;TEND=false;DONE=false;lofile=false;
  // we start at the top of the file
  file_top = true;

  // set the input file
  if (inputFile == "") input_file = "endf.prepro";
  else input_file = inputFile;

  //Here we go...
  Efile.open(input_file.c_str(),ios::in); // Open the preprocessed ENDF file stream
  
  if ( !Efile.is_open() )             // opened failed 
    { FatalError("b6stream::b6stream","Cannot find "+input_file+" file",false); }
  else
    { Info("",input_file+" opened successfully!"); }

  //Loop through the entire file, reading one line at a time
  for ( int i=0; !DONE; i++ ) { readENDF(); }  
  
  Efile.close();  //close the ENDF file stream
  
}
// -------------------- b6stream destructor ----------------
//default destructor
b6stream::~b6stream() 
{ 
  
}
// -------------------- b6stream::readENDF ----------------
void b6stream::readENDF()
{
  //First, read in the line
  if (!getline( Efile, sBuff )) {DONE=true;return;}

  // initialize everything before processing any lines
  if ( ( file_top ) && ( NS == -1 ) )
  {
    SEND=false;FEND=false;MEND=false;TEND=false;
    NTAPE=0; MAT=0; 
    
    //Need to open the "comments" output file
    //This is always the first thing in the ENDF file
    oFile.open(endf_file_name(1,451).c_str(),ios::out);
    if ( !oFile.is_open() )  //open failed
    {
        FatalError("b6stream::readENDF",
    	    "File "+endf_file_name(1,451)+" not opened");
    }
    else		  //open successful -> start writing
    {
      Info("","File "+endf_file_name(1,451)+" opened for write..."); 
      oFile << sBuff << endl;
      lofile=true;  //lofile true when there is an open output file
      prev_mf = 1;
      prev_mt = 451;
    }
  }//end of if ( NS = -1 )

  get_MAT_MF_MT_NS();  //convert the last four columns and strip them  

  // Set some flags if this is a "special" record
  if ( MT == 0 && MF != 0 && MAT != 0 ) SEND=true;
  if ( MT == 0 && MF == 0 && MAT != 0 ) FEND=true;
  if ( MT == 0 && MF == 0 && MAT == 0 ) MEND=true;
  if ( MT == 0 && MF == 0 && MAT ==-1 ) TEND=true;
  
  // Several lines of the comments section are important...
  if( file_top )
  {
    if ( NS == 0 ) //TPID is the first line of the file
    { 
      if ( MF != 0 || MT != 0 ) //Both are zero ONLY on first line
	{                       //This is just a double check
	  Warning("b6stream::readENDF","MF or MT not zero in first line of input!");
	}
      
      NTAPE=MAT; MAT=0; //Tape number, not MAT, is reported on line 0
    }//end of if ( NS = 0 )

    else if ( NS == 1 ) 
    {
      check_mf_mt( );
      read_idiiii(&sBuff,&ZA,&AWR,&LRP,&LFI,&NLIB,&NMOD);
      oFile << sBuff << endl;
    }//end of if ( NS = 1 )

    else if ( NS == 2 )
    {
      check_mf_mt( );
      read_ddiiii(&sBuff,&ELIS,&STA,&LIS,&LISO,&iDummy,&NFOR);
      oFile << sBuff << endl;
    }//end of if ( NS = 2 )

    else if ( NS == 3 )
    {
      check_mf_mt( );
      read_ddiiii(&sBuff,&AWI,&dDummy,&iDummy,&iDummy,&NSUB,&NVER);
      yi=NSUB_to_yi(NSUB);
      oFile << sBuff << endl;
    }//end of if ( NS = 3 )

    else if ( NS == 4 )
    {
      check_mf_mt( );
      read_ddiiii(&sBuff,&TEMP,&dDummy,&LDRV,&iDummy,&NWD,&NXC);
      oFile << sBuff << endl;
    }//end of if ( NS = 4 )

    else if ( NS == 5 )  //get the date, maybe
    {
      check_mf_mt( );
      date = 0;
      if (Global.Value("skip_date") == 0.) read_date(&sBuff,&date);
      else Info("b6stream::readENDF","skipping ENDFB6 evaluation date read.");
      oFile << sBuff << endl;
      // Test to make sure everything seems okay with this evaluation
      // based on the comments file information and print a summary.
      print_summary( );
      // We are done with the first 6 lines
      file_top = false;      
    }//end of if ( NS = 5 )
    else //rest of top data
    {
      check_mf_mt( );
      oFile << sBuff << endl;
    }
  }

  else if ( TEND )
  {
    DONE=true;  // last line of endf.prepro file
  }
  else if ( MEND )
  {
    DONE=true;  // no more data for this mf number
  }
  else if ( FEND )
  {
    // no more data for this mf-mt combination; file should be closed
    if ( lofile )
    {
      Warning("b6stream::readENDF","Closing file with FEND");
      oFile.close();
      lofile=false;
    }
  }
  else if ( SEND )
  {
    // end of this mf-mt data; close the current file
    if ( lofile )
    {
      oFile.close();
      lofile=false;
      Info("b6stream::readENDF","File closed.");
    }
  }
  else if ( !lofile )
  {
    // this is new mf-mt data
    // store the mf and mt numbers
    iMF_array_list::iterator this_mt;
    if( MT_MF_list.at( MT, this_mt ) )
    {
      // this mt number is already in the list, so append this mf
      this_mt->push_back( MF );
    }
    else
    {
      iMF_array_link new_link; // to make a new link
      new_link.MT( ) = MT;
      MT_MF_list.insert( MT_MF_list.end( ), new_link );
      this_mt = MT_MF_list.end( );
      --this_mt;
      // save this mf
      this_mt->push_back( MF );
    }

    // We need to open a new file, test and then send
    oFile.open(endf_file_name(MF,MT).c_str(),ios::out); 
    if ( !oFile.is_open() )  //open failed
    { 
      SevereError("b6stream::readENDF",
             "File "+endf_file_name(MF,MT)+" not opened.");
    }
    else            //open successful
    { 
      Info("b6stream::readENDF","File "+endf_file_name(MF,MT)+
           " opened for write..."); 
      oFile << sBuff << endl;
      lofile=true;
      prev_mf = MF;
      prev_mt = MT;
    }
  }
  else //rest of data
  {
    check_mf_mt( );
    oFile << sBuff << endl;
  }
  
  //Reset these after line has been handled
  SEND=false;FEND=false;MEND=false;TEND=false;
  
}
// -------------------- b6stream::check_mf_mt ----------------
void b6stream::check_mf_mt( )
{
  if( ( MF != prev_mf ) || ( MT != prev_mt ) )
  {
    cout << "MF: " << MF << " prev_mf: " << prev_mf <<
      "  MT: " << MT << " prev_mt: " << prev_mt << endl;
    SevereError( " b6stream::check_mf_mt", " inconsistent mf-mt numbers" );
  }
}
// -------------------- b6stream::get_MAT_MF_MT_NS ----------------
void b6stream::get_MAT_MF_MT_NS()
{
  // Read the MAT, MF, MT and NS into memory, strip them from string buffer
  MAT=stoi(sBuff.substr(66,4));
  MF=stoi(sBuff.substr(70,2));
  MT=stoi(sBuff.substr(72,3));
  if (sBuff.size()>75) NS=stoi(sBuff.substr(75,5));
  else {
    if (NS>=0) NS++;
    else NS=0;
  }
  sBuff = sBuff.substr(0,66) ;
}
// -------------------- b6stream::NSUB_to_yi ----------------
int b6stream::NSUB_to_yi(int NSUB){
    if ( ( NSUB>3     )&&( NSUB<10    ) ) return 0; // activation data
    if ( ( NSUB>9     )&&( NSUB<13    ) ) return 1; // neutrons
    if ( ( NSUB>10009 )&&( NSUB<10020 ) ) return 2; // protons
    if ( ( NSUB>10019 )&&( NSUB<10030 ) ) return 3; // deuterons
    if ( ( NSUB>10029 )&&( NSUB<10040 ) ) return 4; // tritons
    if ( ( NSUB>20029 )&&( NSUB<20040 ) ) return 5; // 3He's
    if ( ( NSUB>20039 )&&( NSUB<20050 ) ) return 6; // alphas
    if ( ( NSUB>=0    )&&( NSUB<4     ) ) return 7; // gammas
    SevereError("b6stream::NSUB_to_yi",
        pastenum("Illegal Sublibrary designator NSUB=",NSUB)
        +", Cannot compute yi");  
    return -1; // should never get here 
}
// -------------------- b6stream::print_summary ----------------
void b6stream::print_summary()
{ //Print a short summary from the comments section header
  if (MESSAGELEVEL_INFO) cout<<"Info :  b6stream summary :"<<endl
    <<"   ENDF Tape number " << NTAPE << endl
    <<"   Library version "<<NVER<<"; identifier "<<NLIB<<endl
    <<"   Library format = "<<NFOR<<endl
    <<"   ZA = "<<ZA<<"  AWR = "<<AWR<<endl;
  if ( LRP==-1 )
    {if (MESSAGELEVEL_INFO) cout<<"   No File 2 given and LRP = -1 (we can't use it anyway...)"<<endl;}
  else if (LRP==0)
    {if (MESSAGELEVEL_INFO) cout<<"   File 2 contains effective scattering radius..."<<endl;}
  else if (LRP==1)
    {/*This is okay */}
  else if (LRP==2)
    {if (MESSAGELEVEL_INFO) cout<<"   LRP = 2 :for use with derived files only"<<endl;}
  else
    {SevereError("b6stream::print_summary","LRP out of range!");}
  if (ZA/1000 > 88) {
    if (LFI==1) if (MESSAGELEVEL_INFO) cout<<"   Material fizzes."<<endl;
    else Warning("b6stream::print_summary",
            pastenum("LFI = ",LFI)+", but material should fiz.");
  } else {
    if (LFI==0) if (MESSAGELEVEL_INFO) cout<<"   Material does not fizz."<<endl;
    else SevereError("b6stream::print_summary",
            pastenum("LFI = ",LFI)+", but material cannot fiz!");
  }
  if (STA==0)
    {if (MESSAGELEVEL_INFO) cout<<"   Target is stable."<<endl;}
  else
    {if (MESSAGELEVEL_INFO) cout<<"   Target is unstable."<<endl;}
  if (MESSAGELEVEL_INFO) cout
    <<"   Incident particle (yi) = "<<yi<<endl
    <<"   Excitation energy =      "<<ELIS<<endl
    <<"   Nucleus state number =   "<<LIS<<endl
    <<"   Isomeric state number =  "<<LISO<<endl
    <<"   Temperature =            "<<TEMP<<endl;
}
// -------------------- b6stream::print_MT_MF ----------------
void b6stream::print_MT_MF( )
{
  cout << "Info: Data for each reaction:"<<endl;
  for( iMF_array_list::iterator this_mt = MT_MF_list.begin();
       this_mt != MT_MF_list.end(); ++this_mt)
  {
    this_mt->print_MF( );
  }
}
