/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1840 $
 * $Date: 2006-04-14 16:26:58 -0700 (Fri, 14 Apr 2006) $
 * $Author: hedstrom $
 * $Id: bdfls_tools.cpp 1840 2006-04-14 23:26:58Z hedstrom $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

//Implementation of the bdfls class

#include "bdfls_tools.hpp"     //converting strings to numbers
#include "convert.hpp"         // converting strings to numbers
#include "global_params.hpp"   //global parameters
#include "messaging.hpp"

extern GlobalParameterClass Global;

// *********** mass_life_link class ***********************
// ----------- mass_life_link::print ---------------
// print the data in a link
void mass_life_link::print( )
{
  cout << za_ << "  " << mass_ << "  " << lifetime_ << endl;
}

// *********** mass_life_list class ***********************
// ----------- mass_life_list::at ---------------
// Function to get the link at "x"
// Returns "false" if the entry is not found.
bool mass_life_list::at(int x, mass_life_list::iterator& link)
{
  for( link = begin(); 
       link != end();
       ++link )
  {
    if ( link->ZA( ) == x ) 
    {
      return true;
    }
  }
  Warning("mass_life_list::at",pastenum("could not find a match for ZA = ",x));
  return false;
}
// ----------- mass_life_list::print ---------------
void mass_life_list::print()
{
  for( mass_life_list::iterator link = begin(); 
       link != end();
       ++link)
  {
    link->print( );
  }
}

// *********** bdflsClass class ******
// -------------------- bdflsClass constructor ----------------
//default constructor
bdflsClass::bdflsClass()   
{
  //The section array holds the names of types of info in the bdfls file
  //in the order they are encountered.
  section[0]="group";
  section[1]="flux";
  section[2]="mass";
  section[3]="lifetime";
  section[4]="nuclear constant";
  section[5]="temperature set";
  section[6]="subshell designator";

  nsection_max=7;
  nsection=0;
  lastsection=-1;
  nc_index=0;

}
// -------------------- bdflsClass destructor ----------------
//default destructor
bdflsClass ::~bdflsClass()
{ 
}
// -------------------- bdflsClass::read ----------------
//Function to read in the bdfls file
void bdflsClass::read( )
{
  char* bdfls_filename="./bdfls";

  if (getenv("BDFLSPATH")!=0) 
  {
    bdfls_filename=getenv("BDFLSPATH");
  }

  string infostring("Opening bdfls file: ");
  infostring+=bdfls_filename;
  
  Info("",infostring );
  
  bdflsFile.open(bdfls_filename,ios::in);

  if ( !bdflsFile.is_open() ) 
  {
    FatalError("bdflsClass::read","Error opening bdlfs file.");
  }

  read_sections();

  Info("","Closing bdfls file ...");
  bdflsFile.close();
}
// -------------------- bdflsClass::read_sections ----------------
//Function that reads each section of the bdfls file
void bdflsClass::read_sections( )
{
  //Reads the entire bdfls file
  string small_string;
  double mass, life;
  int za;
  mass_life_link XYdata;
  mass_life_list::iterator XYptr;
  bool mass_done = false;
  bool life_done = false;
  vector<string> split_line;
  
  while( nsection < nsection_max )  //loop through all the sections
    {
      if ( nsection != lastsection )  //print some info each new section
	{
	  Info("bdflsClass::read_sections","Reading "+section[nsection]+" information...");
	  lastsection=nsection;
	}
      
    //Do what we gotta do for each section of data
    //Decided to use switch statement cuz I have never used them before
    switch( nsection )
	{  
	  
	case 0:
	  if ( !EndofSection() )  //Read & test the line for end of section
	  { //don't care about group data
	  }
	  else
	  { ++nsection;}
	  break;
	  
	case 1:
	  if ( !EndofSection() )  //Read & test the line for end of section
	  { //don't care about flux data
	  }
	  else
	  { ++nsection;}
	  break;
	  
	case 2:
	  if ( !EndofSection() )  //Read & test the line for end of section
	  { //here are the masses
        split_line = split(stringbuff);
        za = stoi(split_line[0]);
        mass = stod(split_line[1]);
        
	    //small_string = stringbuff.substr(0,10);
	    //za = stoi(small_string);  
	    //small_string = stringbuff.substr(11,21);
	    //mass = stod(small_string);
	    
        // have we already got a lifetime for this ZA?
        if(life_done && Mass_Life.at(za, XYptr))
	    {
           XYptr->Mass() = mass;
	    }
           else
	    {
	      XYdata.ZA() = za;  //Load into link
	      XYdata.Mass() = mass;
          XYdata.LifeTime() = 1.0e50;
	      Mass_Life.insert(Mass_Life.end(), XYdata);
	    }
	  }
	  else
	  {
        ++nsection;
        mass_done = true;
      }
	  break;
	  
	case 3:
	  if ( !EndofSection() )  //Read & test the line for end of section
	  { //here are the lifetimes
	    small_string = stringbuff.substr(0,10);
	    za = stoi(small_string);
	    if ( stringbuff.length() < 11 )  // Some have no life info
	    {
	      //Load up a long lifetime if there is a mass entry for this isotope
          if( Mass_Life.at(za, XYptr))
          {
            XYptr->LifeTime() = 1.e50;
          }
	    }                     
	    else
	    {
	      small_string = stringbuff.substr(11,21);
	      small_string = remove_all_blanks( small_string);
	      if ( small_string.length() == 0 ) 
          {
            life = 1.e50;
          }
	      else
          {
            life = stod(small_string);
          }
          // have we already got a mass for this ZA?
          if(mass_done && Mass_Life.at(za, XYptr))
          {
            XYptr->LifeTime() = life;
          }
          else
          {
            XYdata.ZA() = za;  //Load into link
            XYdata.Mass() = -1.0;
            XYdata.LifeTime() = life;
            Mass_Life.insert(Mass_Life.end(), XYdata);
          }
	    }
	  }
	  else
      { ++nsection;}
	  break;
	  
	case 4:
	  if ( !EndofSection() )  //Read & test the line for end of section
      { //here are the nuclear constants
	    if ( nc_index > 0 ) //First line is nothing
		{
		  small_string=stringbuff.substr(0,16);
		  nuclear_constants[nc_index]=stod(small_string);
		}
	      nc_index=nc_index+1;
      }
	  else
      { ++nsection;}
	  break;
	  
	case 5:
	  if ( !EndofSection() )  //Read & test the line for end of section
	  { //don't care about temperature sets data
	  }
	  else
	  { ++nsection;}
	  break;
	  
	case 6:
	  if ( !EndofSection() )  //Read & test the line for end of section
	  { //don't care about subshell designator data
	  }
	  else
	  { ++nsection;}
	  break;
	}
  }
}

// --------------------  bdflsClass::EndofSection ----------------
//! Function that reads in a line of the bdfls file and determines
//! whether or not the line is the last of a section.
bool bdflsClass::EndofSection( )
{
  // Performs two tasks - reads in a line from file and then
  // tests to see if it is the end of a section
  // Note: line containing end of line marker has no other
  // particularly useful information in it (if any other at all)
  
  getline( bdflsFile, stringbuff ); //read in the line
  
  if ( stringbuff.length() >= 73 && stringbuff[72]=='1' )  // end of section mark
  {
    return true;
  }
  else
  {
    return false;
  }
}


