/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: bdfls_tools.hpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

#ifndef BDFLS_TOOLS
#define BDFLS_TOOLS

#include <stdio.h>             // standard I/O package
#include <fstream>             // standard file streams
#include <iostream>            // for file I/O functions
#include <iomanip>             // I/O formatting
#include <string>              // for string fun
#include <cmath>               // math library
#include <list>                // using linked lists

using namespace std;

// ------------------------ class mass_life_link ---------------
//!A link to hold ZA, mass, and lifetime
class mass_life_link
{
private:
  int za_;
  double mass_;
  double lifetime_;

public:
  //!Common name used to access the ZA value of this list.
  inline int& ZA()
  {
    return za_;
  }

  //!Common name used to access contents of the first double in the pair.
  inline double& Mass( )
  {
    return mass_;
  }

  //!Common name used to access contents of the second double in the pair.
  inline double& LifeTime( )
  {
    return lifetime_;
  }

  void print( );
};

// ------------------------ class mass_life_list ---------------
//!A list of mass_life links
class mass_life_list : public list< mass_life_link >
{
public:
  //! Function to get the link at "x".
  // Returns "false" if the entry is not found.
  bool at(int x, mass_life_list::iterator& link);

  void print( );
};

// ------------------------ class bdflsClass ---------------
//! Class to read in and store the bdfls file information.
class bdflsClass
{
  // Only three of the sets of information is interesting for us -
  // the masses, the lifetimes and the nuclear constants.
  // The class was designed as if all data is to be stored but
  // I didn't finish all the sections.  Anyone interested in
  // having any of the other info availabe will have to write
  // some code in the appropriate switch case.
  
public:

  //! Nuclear constants are stored in this array.
  double nuclear_constants[20];

  //! Masses and lifetimes stored in a list of mass_life_links as function of ZA.
  mass_life_list Mass_Life;

  //! Default constructor.
  bdflsClass();  

  //!Default destructor.
  ~bdflsClass(); 

  //! Opens bdfls, loops over sections and then closes bdfls file.
  void read();

private:

  string stringbuff;                     //Temporary string buffer
  string section[10]; 
  int nsection,lastsection,nsection_max; //Section counters and such
  int nc_index;                          //index used while reading in nuclear constants

  //!bdfls ifstream object.
  ifstream bdflsFile; 

  //! Reads in each section of the bdfls file.
  void read_sections();
  
  //! Read in a line and test it for end-of-section.
  bool EndofSection();

};

#endif



