/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1781 $
 * $Date: 2006-03-22 09:55:41 -0800 (Wed, 22 Mar 2006) $
 * $Author: dbrown $
 * $Id: convert.hpp 1781 2006-03-22 17:55:41Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// prototypes of conversion functions

#ifndef CONVERT_UTILITIES
#define CONVERT_UTILITIES

#include <string>
#include <cmath>
#include <iostream>
#include <vector>

using namespace std;

//! A function that converts a string to a double precision number.
double stod( string instring );

//! A function that converts a string to a single precision number.
float stof( string instring );

//! A function that converts a string to an integer.
int stoi( string instring ); 

//! A function that removes the leading blanks of a string
string remove_leading_blanks( string instring );

//! A function that removes the trailing blanks of a string
string remove_trailing_blanks( string instring );

//! A function that removes extra blanks in a string
string remove_extra_blanks( string instring );

//! A function that removes all blanks in a string
string remove_all_blanks( string instring );

//! splits a string at all occurances of "pattern" and returns 
//! the substrings in a list 
vector< string > split(const string& s, const string pattern=" ");

//! joins a list of strings using "pattern" as glue
string join(const vector< string >& ls, const string pattern=",");

//! A function that lower-cases strings
string tolower(const string& s);

//! i copies of a string x pasted together
string operator*(int i, const string x);

//! center a string in a field of a certain length
string center(const string stuff, int len);

//! left justify a string in a field of a certain length
string ljust(const string stuff, int len);

//! right justify a string in a field of a certain length
string rjust(const string stuff, int len);

#endif

