/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1843 $
 * $Date: 2006-04-20 11:38:46 -0700 (Thu, 20 Apr 2006) $
 * $Author: dbrown $
 * $Id: distrib_base.hpp 1843 2006-04-20 18:38:46Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// header for the classes for energy and angle distributions

#ifndef DISTRIB_BASE_CLASS
#define DISTRIB_BASE_CLASS

#include "list_1d.hpp"
#include "global_params.hpp"

using namespace std;
extern GlobalParameterClass Global;

// -------------------- class distrib_base ------------------
//! Base class for one link in the lists used in energy and angle distributions.

class distrib_base : public dd_list
{
protected:
  bool debug_on;  // for checking the 2-d interpolation
  double tol_1d;  // accuracy of linear interpolation
  double cut_off; // don't impose accuracy below this value

  // the maximum probability, used in the tolerance check
  double max_P;

public:
  vector<double> equi_prob;  // the equally probable bins
  double weight;   // The weights add to 1 for multiple models
  int num_bins;
  int interp_type; // type of interpolation between this list and the previous
  bool bin_interp; // if true, use the equi-probable bins to check
            // interpolation, otherwise use unit-base interpolation

  // other usual names for the tag
  inline double& mu()
  {
    return tag;
  }
  inline double& E_gamma()  // for MF13 files
  {
    return tag;
  }

  // debug_on = true;  // for checking the 2-d interpolation

  //!default constructor
  distrib_base(void): debug_on(false), tol_1d(Global.Value("tol_1d")), 
    cut_off(Global.Value("cut_off_1d")), max_P(1.0), 
    equi_prob(static_cast<int>(Global.Value("num_mf5_bins"))+1,0.0), 
    weight(1.0), 
    num_bins(static_cast<int>(Global.Value("num_mf5_bins"))), 
    interp_type(0), bin_interp(true){}

  // *** start of virtual functions ***
  // the distribution function---it depends on the model
  virtual double f(double E) = 0;
  // *** end of virtual functions ***

  // Interpolate this list between left_list and right_list
  // using the equiprobable bins.
  void list_interp(double e_in, distrib_base& left_list,
    distrib_base& right_list);

  void print();

  // Expand part of a list to achieve the tolerance
  void thicken( dd_list::iterator first, dd_list::iterator last );

  // Expand the entire list to achieve the tolerance
  void thicken( );

  // get the probability for this list.
  // This routine is used for double differential data
  double get_norm();

  // ensure that the total probability is 1
  void renorm();

  // renormalize with the norm given
  void renorm(double Norm);

  // make the equally probable energy bins
  void get_bins();

  //mirror a distribution in energy
  void mirror();

  // This routine widens the delta functions for the whole list
  void widen_delta( );

  // Check the accuracy of 2d linear interpolation based on the
  // equally probable bins.
  bool check_equiprob(distrib_base& left_list,
    distrib_base& right_list, double tol_2d);

  // This routine was once used to check unit-based interpolation.
  bool check_interp_2d(double tol_2d, double cut_off_2d);

};
#endif



