/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: global_params.hpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

#ifndef GLOBAL_PARAMS
#define GLOBAL_PARAMS

#include <stdio.h>             // standard I/O package
#include <fstream>             // standard file streams
#include <iostream>            // for file I/O functions
#include <sstream>
#include <iomanip>             // I/O formatting
#include <string>              // for string fun
#include <cmath>               // math library
#include <list>

using namespace std;

#define MESSAGELEVEL_INFO  Global.Value("message_level")==0.0
#define MESSAGELEVEL_WARN  Global.Value("message_level")<=1.0
#define ENDL_DATA_PRECISION  (int)(Global.Value("endl_datafield_precision"))

// ----------------------- class ss_link -------------------
//! Links for the list of input parameters
class ss_link
{
private:
  string x;  // the variable name
  string y;  // the value of the variable

public:
  // default constructor
  ss_link( )
  { }

  // default deonstructor
  ~ss_link( )
  { }
   
  //!Common name used to access the X value of this list.
  inline string& name( )
  {
    return x;
  }

  //!Common name used to access the Y value of this list.
  inline string& value( )
  {
    return y;
  }

  //! Method for printing this link.
  void print( );

};

// ----------------------- class ss_list -------------------
//! List of ss_links.
class ss_list : public list< ss_link >
{
public:
  //! Function to get the link at "x".
  // Returns "false" if the entry is not found.
  bool at(string x, ss_list::iterator& link);

  void print( );

};

// ----------------------- class GlobalParameterClass -----------
//! Read/store the global parameter settings for the translation
class GlobalParameterClass
{
public:
  //!Default constructor - initializes Parameter list and values
  GlobalParameterClass();

  //!Default destructor.
  ~GlobalParameterClass();

  //!The function used to input information into the Parameter list.
  void set( string gp, double gp_num);

  //!The function used to input information into the Parameter list.
  void set( string gp, string gp_val);

  //! Function to print a table of parameters and values used in the translation.
  void print();

  //! Function to parse command line arguments and update Parameter list.
  string read_command_line(int argc, char* argv[] );

  //! Function to read in a parameter file and update the Parameter list.
  void read_file( string file );

  //!Delivers the stored value of the requested parameter.
  double Value( string ParamName );   

  //!Delivers the string representation of the requested parameter.
  string Flag( string ParamName );   

private:

  //! Local string buffer
  string spar[50];

  //!Parameters stored in name indexed link list
  ss_list Parameters;

};

#endif



