/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1896 $
 * $Date: 2006-09-07 14:49:40 -0700 (Thu, 07 Sep 2006) $
 * $Author: dbrown $
 * $Id: load_data.cpp 1896 2006-09-07 21:49:40Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// methods

#include "load_data.hpp"
#include "kalbach.hpp"
#include "record_types.hpp"
#include "endl_formats.hpp"
#include "mf4classes.hpp"
#include "mf5classes.hpp"
#include "mf6classes.hpp"
#include "mf6_gammas.hpp"
#include "mf12classes.hpp"
#include "mf13classes.hpp"
#include "mf14classes.hpp"
#include "mf14table.hpp"
#include "messaging.hpp"
#include "multiplicity.hpp"
#include "charge.hpp"

extern ENDLClass ENDL;
extern GlobalParameterClass Global;
extern C55_gammas C55;

// ---------------------- LoadData ------------------
void LoadData( int F, int T )
{
  try {
    // Do what we need to do based on the type of data...
    if ( T==5 )  // general particle production
    {
            Unimplemented("LoadData",
            "general particle production has no ENDL equivalent.");
    } 
    else if ( F==1 )
    {
        // T = 451,... is for delayed fission neutrons
        if ( T < 400 )   // general information
        {
            SevereError("LoadData",pastenum("MF = ",F)+" not used.");
        }
    }
    else if ( F==2 )  //Resonance Parameter data
    {}
    else if ( F==3 )  //Reaction Cross sections
    {
      mf3_file inFile;   //Instantiate the input file
      inFile.open(F, T);
      LoadMF3Data( inFile );
      Info("LoadData","Closing file "+endf_file_name(F,T)+"...");
      inFile.close();
    }
    else if( !ENDL.write_file )
    {
      // ignore this data
      return;
    }
    else if ( F==4 )  //Ang Dists for emmitted particles
    {
      mf4_file inFile;   //Instantiate the input file
      inFile.open(F, T);
      LoadMF4Data( inFile );
      Info("LoadData","Closing file "+endf_file_name(F,T)+"...");
      inFile.close();
    }
    else if ( F==5 )
    {
      mf5_file inFile;   //Instantiate the input file
      inFile.open(F, T);
      LoadMF5Data( inFile );
      Info("LoadData","Closing file "+endf_file_name(F,T)+"...");
      inFile.close();
    }
    else if ( F==6 )  //E-Ang Dists for emitted particles
    {
      mf6_file inFile;   //Instantiate the input file
      inFile.open(F, T); //Open the file
      LoadMF6Data( inFile ); //Load and translate the data
      Info("LoadData","Closing file "+endf_file_name(F,T)+"...");
      inFile.close();
    }
    else if ( F==7 )  //Thermal Neutron scattering Law
    {
        Unimplemented("LoadData",pastenum("MF = ",F)+" not implemeted yet.");
    }
    else if ( F==8 )  //Radio and fission-product yield data
    {
        Info("LoadData",
            "ENDL does not support Radio and fission-product decay data, "+
            pastenum("MF = ",F));
    }
    else if ( F==9 )  //Multiplicities for radio nuclide prod
    {
        Unimplemented("LoadData",
            "fete does not yet translate Multiplicities for radio nuclide prod, "+
            pastenum("MF = ",F));
    }
    else if ( F==10 ) //Cross sections for radio nuclide prod
    {
        Unimplemented("LoadData",
            "fete does not yet translate Cross sections for radio nuclide prod, "+
            pastenum("MF = ",F));
    }
    else if ( ( F==12 ) || ( F==13 ) )  //Multiplicities for photon production
    {
        if( Global.Value( "skip_gammas" ) <= 0.0 )
        {
            if( T > 4 ) // gammas for individual reactions
            {
                mf12_file inFile;   //Instantiate the input file
                inFile.open(F, T);
                LoadGammaData( inFile );
                Info("LoadData","Closing file "+endf_file_name(F,T)+"...");
                inFile.close();
            }
            else  //Cross sections for photon production 
            {
                mf13_file inFile;   //Instantiate the input file
                inFile.open(F, T);
                LoadMF13Data( inFile );
                Info("LoadData","Closing file "+endf_file_name(F,T)+"...");
                inFile.close();
            }
        }
    }
    else if ( F==14 ) //Ang Dists for photon production
    {
        // do nothing; mf14 files are treated under mf12 or mf13
    }
    else if ( F==15 ) //energy distributions for continuum photons
    {
        // do nothing; mf15 files are treated under mf12 or mf13
    }
    else if ( F==23 ) //Photo-atomic interaction x-sections
    {
        Warning("LoadData",pastenum("MF = ",F)+" not implemeted yet.");
    }
    else if ( F==27 ) //Atomic form facts for photo-atomic int
    {
        Warning("LoadData",pastenum("MF = ",F)+" not implemeted yet.");
    }
    else if ( F==30 ) //Data covariances obtained from 
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }                           //parameter covariances and sensitivities
    else if ( F==31 ) //Data cov for nu(bar)
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }
    else if ( F==32 ) //Data cov for resonance params
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }
    else if ( F==33 ) //Data cov for reaction x-sections
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }
    else if ( F==34 ) //Data cov for angular dists
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }
    else if ( F==35 ) //Data cov for energy dists
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }
    else if ( F==39 ) //Data cov for radionuke prod yields
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }
    else if ( F==40 ) //Data cov for radionuke prod x-sections
    {
        Unimplemented("LoadData","ENDL does not support covariances yet, "
            +pastenum("MF = ",F)+" not implemented.");
    }
    else //We have a problem...
    {
        SevereError("LoadData",pastenum("MF = ",F)+" not valid.");
    }
  }
  catch (string str) {
    cerr << "fete LoadData: " << str <<
            ", abort processing of this file." << endl;
    return;
  }
  // Reset the S number.  If was S = 0, 1, or 9 this is harmless.  
  // If was set, e.g. for delayed n's to S=7, this will set it back to the correct value.
  ENDL.set_s_number();
  // Recompute the Q value.  If QM or QI had to get reset (as happens when doing fission),
  // we'll need to re-run this.  Otw, this is harmless.
  ENDL.set_QValue();
}

// --------------- LoadMF1Data ------------------------
// Deprecated
void LoadMF1Data( int T, mf1_file& inFile )
{
    Warning( "LoadMF1Data", "Use of this function is deprecated.  Use LoadFissionData instead." );
    LoadFissionData( );
}

// --------------- MTExists ------------------------
bool MF1MTExists( int MT ){
    mf1_file inFile;
    inFile.open( 1, MT, false );
    return inFile;
}

// --------------- LoadFissionData ------------------------
// Data for fission neutrons.  Call this only when 
// translating the MF=3, MT=18 (fission) data.
// In this routine, we will process (or not) MT's 452, 455, 456, 458, 460
void LoadFissionData(  )
{
    int LNU;    // flag: 1 for spontaneous fission, 2 for induced fission
    int ZA;     // target ZA 
    double AWR; // target mass
    bool got_nubar_prompt  = false;
    bool got_nubar_delayed = false;

    // Process the energy release due to fission (MT = 458)
    if ( Global.Value("fission_Q") != 0.0 ) 
    {
        // Do not include energies of delayed fission decays
        if( MF1MTExists( 458 ) ) 
        {
            Info("LoadFissionData","Processing energy release due to fission (MT = 458) file.");
            Info("LoadFissionData","Computing Q values based on (MT = 458) file.");
            mt458_file Q_file;
            Q_file.open( 1, 458 );
            double QM = Q_file.reduce_Q( ENDL.QM/ENDL.eV2MeV );
            ENDL.reaction( 0, QM, QM ); 
            Info("LoadFissionData","Closing file "+endf_file_name(1,458)+"...");
            Q_file.close();
        }
        else Info("LoadFissionData","No delayed decay energy (MT = 458) file.");
    }

    // Set the fission product residual
    ENDL.res_za = 99120; 
    
    // Don't process the post-fission beta-delayed gamma data (MT = 460)
    if ( MF1MTExists( 460 ) ) Info("LoadFissionData","Skipping post-fission beta-delayed gamma data (MT = 460) file.");

    // delayed fission nubar
    if ( MF1MTExists( 455 ) ) 
    {
        Info("LoadFissionData","Processing delayed fission nubar (MT = 455) file.");
        mf1_file inFile;
        inFile.open( 1, 455 );
        inFile.read_line1(&ZA, &AWR, &LNU);
        fission_delay fiss_delay;
        fiss_delay.manager(inFile);
        got_nubar_delayed = true;
        Info("LoadFissionData","Closing file "+endf_file_name(1,455)+"...");
        // Reset the ENDL object so can finish the cross section processing
        ENDL.set_yo( 0 );
        ENDL.set_s_number( 0 ); 
        ENDL.set_x1( 0.0 );
    }
    else Info("LoadFissionData","No delayed fission nubar (MT = 455) file.");

    // prompt fission nubar
    if ( MF1MTExists( 456 ) ) 
    {
        Info("LoadFissionData","Processing prompt fission nubar (MT = 456) file.");
        mf1_file inFile;
        inFile.open( 1, 456 );
        inFile.read_line1(&ZA, &AWR, &LNU);
        multiplicity multiple;
        //read second line
        int NR, NP;
        inFile.mult_head(
          &NR,   // number of INT regions 
          &NP    // number of data points
        );
        inFile.get_regions( NR, multiple.NBT, multiple.INT );
        multiple.read_data( NP, inFile );
        ENDL.set_yo(1);
        if ( ENDL.write_file ) multiple.write_endl( 7 );
        got_nubar_prompt = true;
        Info("LoadFissionData","Closing file "+endf_file_name(1,456)+"...");
        // Reset the ENDL object so can finish the cross section processing
        ENDL.set_yo( 0 );
        ENDL.set_s_number( 0 ); 
    }
    else Info("LoadFissionData","No prompt fission nubar (MT = 456) file.");

    // total fission nubar
    if ( MF1MTExists( 452 ) ) 
    {
        if ( got_nubar_prompt && got_nubar_delayed )
        {
            Info("LoadFissionData","Skipping total fission nubar (MT = 452), it is redundant so not worth processing.");
        }
        else
        {
            Info("LoadFissionData","Processing total fission nubar (MT = 452) file.");
            mf1_file inFile;
            inFile.open( 1, 452 );
            inFile.read_line1(&ZA, &AWR, &LNU);
            multiplicity multiple;
            //read second line
            int NR, NP;
            inFile.mult_head(
                &NR,   // number of INT regions 
                &NP    // number of data points
            );
            inFile.get_regions( NR, multiple.NBT, multiple.INT );
            multiple.read_data( NP, inFile );
            ENDL.set_yo( 1 );
            ENDL.set_s_number( 17 ); // Bogus S number, we'll have to post-process
            if ( ENDL.write_file ) multiple.write_endl( 7 ); 
            Info("LoadFissionData","Closing file "+endf_file_name(1,452)+"...");
            // Reset the ENDL object so can finish the cross section processing
            ENDL.set_yo( 0 );
            ENDL.set_s_number( 0 ); 
        }
    }
    else Info("LoadFissionData","No total fission nubar (MT = 452).");

}

// --------------- LoadMF3Data ------------------------
void LoadMF3Data( mf3_file& inFile ) 
{
  // Instantiate the cross section object
  one_d_table xs;  

  int ZA;
  double AWR;
  double QM, QI;
  int LR,NR, NP;

  // Get the target ZA and mass
  inFile.read_line1(&ZA, &AWR);

  // Read second line
  inFile.read_line2(
	      &QM,             // mass difference Q value
	      &QI,             // reaction Q value for the lowest state
	      &LR,             // complex or "breakup" flag
	      &NR,             // number of INT regions 
	      &NP              // number of data points
  );
  
  // Stash the reaction info
  ENDL.reaction( LR, QM, QI ); 

  // Handle all fission neutron data except the cross-section
  if ( ENDL.T == 18 ) LoadFissionData(  );  

  if ( !ENDL.write_file )
  {
    Info("LoadMF3Data",pastenum("Skipping mf: ",ENDL.F)+pastenum(" mt: ",ENDL.T));
    return;
  }

  // Red's code has already handled the multiple region representations
  // so we should never see more than one region in the cross sections
  if ( NR != 1 ) SevereError("LoadMF3Data","Expected single region, was this file preprocessed?");
  
  inFile.get_regions( NR, xs.NBT, xs.INT );
  
  xs.read_data( NP, inFile, ENDL.Max_E_in );

  if ( ENDL.write_file ) 
  {
    // Clean up interpolation table
    xs.unique();  
    xs.widen_jumps(); 
    xs.write_endl( 0 );
  }

  // Calculate the compund nucleus and the residual ZA 
  // (For fission, we did this already in the LoadFissionData() routine)
  if( ENDL.T != 18 ) ENDL.get_resid( );
}

// ------------------- LoadMF4Data -------------------------------
void LoadMF4Data( mf4_file& inFile ) 
{
  int ZA, LVT, LTT;
  int LI, LCT;
  int NK,NM;
  double AWR;
  int ZAP, NR, NE;

  inFile.read_line1(
          &ZA,              // target Z & A 
	      &AWR,             // target mass
	      &LVT,             // transformation matrix flag
	      &LTT              // representation flag
  );
  inFile.read_line2(
	      &LI,              // 1 -> all ang dists are isotropic
	      &LCT,             // 1 -> lab frame, 2 -> CM system
	      &NK,              // number of elements in transformation matrix 
	      &NM               // max order of Legendre (should be even)
  );
  
  // check whether the data is in the right frame
  if( ( ENDL.angles_CM && ( LCT != 2 ) ) ||
      ( !ENDL.angles_CM && ( LCT != 1 ) ) )
  {
    Warning( "LoadMF4Data", " wrong frame " + pastenum("LCT = ",LCT) ); 
    // we can handle it.  it might be crap, but we can handle it.
    //return;
  }

  if( LVT == 1)
  {
    // The next NK numbers give the transformation matrix; skip it
    inFile.skip(NK);
  }

  //Stash the reaction information
  ZAP=ENDL.get_ZAP( ENDL.T );
  ENDL.set_outgoing_ZA( ZAP );

  if ( !ENDL.write_file )
  {
    Info("LoadMF4Data",pastenum("Skipping mf: ",ENDL.F)+pastenum(" mt: ",ENDL.T));
    return;
  }

  //There are several possibilities as to what is in this file

  //Start reading and translating. First the isotropic distributions
  if ( LTT==0 )                //all angular distributions are isotropic
  {
    two_d_isotropic isotropic;
    if ( LI != 1 )   //supposed to be isotropic. if not, send message
    {
      Unimplemented("LoadMF4Data","Need new code for "+pastenum("LTT = ",LTT)
        +pastenum(" LVT = ",LVT)+pastenum(" LI = ",LI));
      return;
    }
    Info("LoadMF4Data","Doing mf4 isotropic");
    isotropic.expand_data( ENDL.T );  // make the data
    if ( ENDL.write_file )
    {
      isotropic.widen_jumps();
      isotropic.write_endl(1); //write endl
      if ( ( ENDL.res_za <= 2004 ) && ( ENDL.S != 9 ) )
      {
        ENDL.outgoing_particle = 10 + ENDL.za_to_yo( ENDL.res_za );
        isotropic.write_endl(1);
      }
    }
  }

  // for non-isotropic distributions there are three options - LTT=1,2,3
  else if  ( LTT==1 )  //Legendre Poly coefficients
  {
    Info("LoadMF4Data","Doing mf4 Legendre");
    two_d_Legendre Legendre;             //Create the Legendre object

    inFile.read_line3(
      &NR,              // number of interpolation regions
      &NE               // number of incident energy points
    );
    
    inFile.get_regions(NR, Legendre.NBT, Legendre.INT);

    Legendre.expand_data( inFile, NE, ENDL.F ); //read/expand data to pointwise
    Legendre.widen_jumps();
    Legendre.renorm();
    if (ENDL.write_file)
    {
      Legendre.write_endl(1); //write the endl file
      // do we have elastic scattering for a light target?
      if ( ( ENDL.T == 2 ) && ( ENDL.res_za <= 2004 ) )
      {
        ENDL.outgoing_particle = 10 + ENDL.za_to_yo( ENDL.res_za );
        Legendre.mirror();
        Legendre.write_endl(1);
      }
    }
  }

  else if ( LTT==2 )  // normalized probability distributions
  {
    Info("LoadMF4Data","Doing mf4 table");
    mf4_table Tabular;    //Create the tabulated object
    Tabular.master( inFile );
    return;
  }
  else if ( LTT==3 ) // new format: like LTT==1 for low E, LTT==2 for high E
  {
    Info("LoadMF4Data","Doing mixed mf4");
    mixed_mf4 mixed_data;
    mixed_data.master( inFile );
  }
  else
  {
    SevereError("LoadMF4Data",pastenum("LTT = ",LTT)
      +" in MF4 data is impossible!");
  }
}

// ------------------- LoadMF5Data -------------------------------
void LoadMF5Data( mf5_file& inFile ) 
{
  int ZA;
  int LF;
  int NK;
  double AWR,U;
  int ZAP,NR, NP;

  // read the first line of the file
  inFile.read_line1(&ZA, &AWR, &NK);

  //Stash the reaction information
  ZAP=ENDL.get_ZAP(ENDL.T);
  ENDL.set_outgoing_ZA( ZAP );

  if ( !ENDL.write_file )
  {
    Info("LoadMF5Data",pastenum("Skipping mf: ",ENDL.F)+pastenum(" mt: ",ENDL.T));
    return;
  }

  if( NK > 1)
  {
    Info("LoadMF5Data","Doing mf5 multiple models");
    add_them_up add_them( NK );
    add_them.manager( inFile );
  }
  else
  {
    // read the second line of the file
    inFile.read_line2(&U, &LF, &NR, &NP);

    if ( LF == 1  ) //Arbitrary tabulated function
    {
      Info("LoadMF5Data","Doing mf5 table");
      mf5_table Arbitrary;            //Create the table
      Arbitrary.read_data(inFile, NR, NP );  //read in the data
      if (ENDL.write_file) 
      {
        if( Arbitrary.duplicate_Ein ){
            Arbitrary.widen_jumps( );
        }
        Arbitrary.renorm(); 
        Arbitrary.write_endl(4); //write the ENDL file
      }
      return;
    }
    else if ( LF == 5  ) //General evaporation Spectrum
    {
      Unimplemented("LoadMF5Data","Need new code for LF=5");
      return;
    }
    else if ( LF == 7  ) //Simple fission spectrum (Maxwellian)
    {
      Info("LoadMF5Data","Doing mf5 Maxwell");
      two_d_Maxwell Maxwell(U);
      Maxwell.read_data(inFile, NR, NP );  //read in the data
      Maxwell.make_list();   // expand the data
      if (ENDL.write_file) 
      {
        if( Maxwell.duplicate_Ein ){
            Maxwell.widen_jumps( );
        }
        Maxwell.renorm(); 
        Maxwell.write_endl(4); //write the ENDL file
      }
      return;
    }
    else if ( LF == 9  ) //Evaporation spectrum
    {
      Info("LoadMF5Data","Doing mf5 evaporation");
      two_d_evap evap(U);
      evap.read_data(inFile, NR, NP );  //read in the data
      evap.make_list();   // expand the data
      if (ENDL.write_file) 
      {
        if( evap.duplicate_Ein ){
            evap.widen_jumps( );
        }
        evap.renorm(); 
        evap.write_endl(4); //write the ENDL file
      }
      return;
    }
    else if ( LF == 11 ) //Energy-Dependent Watt spectrum
    {
      Info("LoadMF5Data","Doing mf5 Watt");
      two_d_Watt Watt(U);
      Watt.read_data(inFile, NR, NP );  //read in the data
      Watt.make_list();   // expand the data
      if (ENDL.write_file) 
      {
        if( Watt.duplicate_Ein ){
            Watt.widen_jumps( );
        }
        Watt.renorm(); 
        Watt.write_endl(4); //write the ENDL file
      }
      return;
    }
    else if ( LF == 12 ) //Energy dep fission neutron spectrum (Madland & Nix)
    {
      Info("LoadMF5Data","Doing mf5 Madland");
      two_d_Madland Madland;
      Madland.read_data(inFile, NR, NP );  //read in the data
      Madland.make_list();   // expand the data
      if (ENDL.write_file) 
      {
        if( Madland.duplicate_Ein ){
            Madland.widen_jumps( );
        }
        Madland.renorm(); 
        Madland.write_endl(4); //write the ENDL file
      }
      return;
    }
    else
    {
      SevereError("LoadMF5Data",pastenum("There is no model for LF = ",LF));
    }  
  }
}

// ------------------- LoadMF6Data -------------------------------
void LoadMF6Data( mf6_file& inFile ) 
{
  int ZA, LCT, NK;
  double AWR;
  int ZAP, LIP, LAW, NR_1,NR_2, NP;
  double AWP;
  int LANG, LEP, NE;
  // by default make an i=3 gamma file except for neutron capture
  bool make_i3 = ( ENDL.C == 46 )? false : true;
  // Do we write a copy for the ENDL residual as particle?
  bool res_copy = false;
  
  inFile.read_line1(
      &ZA,         // target Z & A 
      &AWR,        // target mass
      &LCT,        // data reference frame
      &NK          // number of subsections
  );
  
  int prev_ZAP = -1;     // no previous outgoing particle
  int X4_count;          // the value of X[4] in (n, Xn) reactions

  for(int iNK = 0; iNK < NK; ++iNK)
  {
    inFile.particle_line1(
      &ZAP,      // product Z & A
      &AWP,      // product mass
      &LIP,      // residual isomeric state
      &LAW,      // kinetics model
      &NR_1,     // number of interpolation regions
      &NP        // number of multiplicity points
    );
    ENDL .set_outgoing_ZA( ZAP );

    // is this the residual?
    if( ( ENDL.current_za == ENDL.res_za ) && ( ZAP == ENDL.res_za ) )
    {
      ENDL.outgoing_particle += 10;
    }

    // this may be gamma data following skipped data for the residual
    if( prev_ZAP == ENDL.res_za )
    {
      ENDL.write_file = true;
    }

    if ( !ENDL.write_file )
    {
      Info("LoadMF6Data", pastenum("Skipping mf: ",ENDL.F)+
        pastenum(" mt: ",ENDL.T)
        +pastenum(" ZAP: ",ZAP));

      if( ( iNK == NK-1 ) || ( ENDL.C < 0 ) ) // this is the last particle
      {
        return;
      }
    }

    if( ZAP == prev_ZAP )
    {
      // This is multiple particle data as for (n, 2n)
      ENDL.append = true;
      ++X4_count;
      ENDL.X[4] = X4_count;
    }
    else
    {
      ENDL.append = false;
      X4_count = 0;
    }

    // Read the multiplicity data first
    multiplicity multiple;
    int this_multiple = 0;  // used in (n, Xn) reactions

    // interpolation regions
    inFile.get_regions(NR_1, multiple.NBT, multiple.INT);

    multiple.read_data( NP, inFile );

    if ( !ENDL.write_file )
    {
      // for the residual as particle we have set ENDL.outgoing_particle = 109
      if( ENDL.outgoing_particle > 100 ) this_multiple = 1;
    }
    else if( ( ZAP == 0 ) && ( LAW > 0 ) )  // for LAW = 0 it's printed later
    {
      // it's a sad state of affairs when I need to make sure 
      // there aren't a negative number of gammas
      multiple.set_min_mult(0.0);
    
      // print the gamma multiplicity
      multiple.write_endl(9);
    }
    else if( ( ENDL.T == 18 ) && ( ZAP == 1 ) )
    {
      // fission neutron multiplicity
      multiple.write_endl(7);
    }
    else if( ENDL.T != 5 )
    {
      // if we already know the multiplicity from the MT number, 
      // let's use that instead if the value in the ENDF file is bogus
      if ( !multiple.check_const_mult( ENDL.yo_mult( ) ) ) 
      {
        Warning("LoadMF6Data", "Found non-constant or incorrect multiplicity for a reaction requiring constant multiplicity, so overwriting");
        multiple.set_mult( ENDL.yo_mult( ) );  
      }
      this_multiple = multiple.get_mult( );
      // we may need to copy the data as residual
      if( ( this_multiple > 1 ) && ( this_multiple*ZAP == ENDL.current_za ) )
      {
        res_copy = true;
      }
      // this could be the first of split (n, Xn) data
      if( ( X4_count == 0 ) && ( this_multiple < ENDL.yo_mult() ) )
      {
        X4_count = 1;
        ENDL.X[4] = X4_count;
      }
    }

    if ( LAW == 0 )  // only multiplicities
    {
      if ( ENDL.write_file )
      {
        multiple.write_endl( 9 );
      }
    }
    else if ( LAW == 1 )  // continuum energy-angle distributions
    {
      // read the next line
      inFile.get_model(
        &LANG,  // the model
        &LEP,   // interpolation for E_out
        &NR_2,  // number of regions
        &NE     // number of incident energies
      );
  
      if(LANG == 1)
      {
        if( ( ZAP == 0 ) && make_i3 )
        {
          Info("LoadMF6Data","Doing i=3 gammas");
          mf6_gammas gammas;
          gammas.read_data( inFile, LEP, NR_2, NE );
        }
        else
        {
          Info("LoadMF6Data",pastenum("Doing mf6 Legendre for ZAP=",ZAP));
          mf6_Legendre Legendre;

          // mcapm currently can't handle mixed double differential data
          if( make_i3 && ( iNK > 0 ) && ENDL.write_file )
          {
            Warning("LoadMF6Data", "mixing types of double differential data");
          }

          Legendre.master( inFile, LEP, NR_2, NE, res_copy );
          if( ENDL.write_file )
          {
            make_i3 = false;
          }
        }
      }
      else if(LANG == 2)
      {
        Info("LoadMF6Data",pastenum("Doing mf6 Kalbach for ZAP=",ZAP));
        //  Kalbach
        if( ( LCT != 2 ) && ( LCT != 3 ) )
        {
          Warning("LoadMF6Data",
            pastenum("Kalbach data must be center-of-mass coordinates.  Evaluation uses LCT=",LCT)+
            ".  Since this makes no sense, we are treating it as center-of-mass data.");
          LCT=2;
        }
        if(LEP != 1)
        {
          SevereError("LoadMF6Data","Implement Kalbach for non-histogram data.");
        }
      
        three_d_Kalbach Kalbach( ZA, ENDL.yo_to_za( ENDL.incident_particle ), ZAP, this_multiple );
        // get the interpolation rules
        inFile.get_regions(NR_2, Kalbach.NBT, Kalbach.INT);
        Kalbach.expand_data(NE, inFile);
        if ( ENDL.write_file )
        {
          Kalbach.renorm();
          Kalbach.write_endl( );
          if( res_copy )
          {
            // make a copy for the residual as particle
            ENDL.outgoing_particle += 10;
            Kalbach.write_endl( );
          }
        }
      }
      else
      {
        Unimplemented("LoadMF6Data",pastenum("LANG = ",LANG)+" not implemented");
      }
    }
    else if ( LAW == 2 )
    {
      // general discrete 2-body
      Info("LoadMF6Data","Doing general mf6 2-body");
      if( LCT != 2 )
      {
        SevereError("LoadMF6Data","Expected center-of-mass coordinates");
      }
      gen_mf4 mu_dist;
      mu_dist.master( inFile );
      make_i3 = false;
    }
    else if ( LAW == 5 )
    {
      Info("LoadMF6Data",pastenum("Doing charged-particle scattering for ZAP=",ZAP));
      two_d_charge mu_dist;
      mu_dist.master( inFile );
    }
    else if ( LAW == 6 )
    {
      // N-body phase-space model
      Info("LoadMF6Data",pastenum("Doing  mf6 N-body phase-space model for ZAP=",ZAP));
      phase_space phase( 1, ZA, ZAP );
      phase.expand_data( inFile, multiple );
      if ( ENDL.write_file )
      {
        phase.write_endl( );
	if( res_copy )
        {
	  // make a copy for the residual as particle
      ENDL.outgoing_particle += 10;
	  phase.write_endl( );
        }
      }
    }
    else if ( LAW == 7 )
    {
      // ENDL-type angle-energy table in laboratory coordinates
      if ( ENDL.write_file )
         Info("LoadMF6Data",pastenum("Doing  mf6 ENDL-type angle-energy table for ZAP=",ZAP));
      if( LCT != 1 )
      {
        SevereError("LoadMF6Data","Expected laboratory coordinates");
      }
      ENDL_table table;
      table.read_data( inFile, ZAP, multiple );
      if ( ENDL.write_file )
      {
        table.write_endl( );
        if( res_copy )
        {
          // make a copy for the residual as particle
          ENDL.outgoing_particle += 10;
          table.write_endl( );
        }
      }
    }
    else if ( ( LAW == 3 ) && 
              (
                ( 
                  ( ENDL.C == 11 ) || 
                  ( ENDL.C == 40 ) ||
                  ( ENDL.C == 41 ) ||
                  ( ENDL.C == 42 ) ||
                  ( ENDL.C == 44 ) ||
                  ( ENDL.C == 45 )
                ) && ( ENDL.S == 1 ) 
              ) 
            )
    {
      //Make an isotropic distribution
      two_d_isotropic isotropic;
      Info("LoadMF6Data",pastenum("Doing mf4 isotropic for ZAP=",ZAP));
      isotropic.expand_data( ENDL.T );  // make the data
      if ( ENDL.write_file )
      {
        isotropic.write_endl(1); //write endl
      }
      make_i3 = false;
    }
    else if ( ( LAW == 4 ) && ( ENDL.current_za == ENDL.res_za ) )
    {
      // This is harmless residual data, just go on
    }
    else
    {
      Unimplemented("LoadMF6Data",pastenum("Implement LAW ",LAW));
      return;
    }
    prev_ZAP = ZAP;
    // update the current residual nucleus
    if ( ENDL.T != 5 )
    {
      ENDL.current_za -= this_multiple * ZAP;
    }
  }
  // set the particle count to zero for the next reaction
  ENDL.X[4] = 0;
}

// --------------- LoadGammaData ------------------------
void LoadGammaData( mf12_file& inFile ) 
{
  if ( !ENDL.write_file )
  {
    Info("LoadGammaData",pastenum("Skipping mf: ",ENDL.F)+pastenum(" mt: ",ENDL.T));
    return;
  }

  int ZA;
  double AWR;
  int LO,LG,mf12_NK;

  ENDL.set_yo( 7 );

  // there may be an mf14 file
  int mf14_LI, mf14_LTT;
  int mf14_NK, mf14_NI;
  mf14_file angle_file;   //Instantiate the input file
  angle_file.open( 14, ENDL.T );
  if( angle_file )
  {
    Info("LoadGammaData","Loading angular distributions from the MF=14 file.");
    // read the first line
    angle_file.first_line(
      &ZA,       // target Z & A 
      &AWR,      // target mass
      &mf14_LI,  // isotropy flag
      &mf14_LTT, // Legendre flag
      &mf14_NK,  // number of discreet photons (inc continuum)
      &mf14_NI   // number of isotropic photon dists
    );
  }
  else
  {
    Info("LoadGammaData","All gammas are emitted isotropicly.");
    mf14_LI = 1;  //all photons isotropic
  }

  inFile.read_line1(
    &ZA,       // target Z & A 
    &AWR,      // target mass
    &LO,       // representation flag (=1 or 2) 
    &LG,       // for LO = 2: flag for gamma source
    &mf12_NK   // number of discrete photons (inc continuum)
  );
  
  // What we do next depends on the LO flag...
  
  // We have multiplicities
  if ( ( LO == 1 ) || ( ENDL.F == 13 ) ) 
  {
  
    //  isotropic discrete gamma energies
    if( mf14_LI == 1 )
    {
      Info("LoadGammaData","Doing MF=12 discrete table");
      MF12_list e_dist;
      e_dist.master( AWR, mf12_NK, &inFile );
    }
    
    //  isotropic discrete+continuum gammas
    else if ( mf14_LTT == 1 )
    {
      // do we split off the discrete lines?
      if( Global.Value( "split_gammas" ) > 0 )
      {
        Info("LoadGammaData","Splitting discrete & continuum gammas");
        MF14_split e_dist;
        e_dist.master( AWR, mf12_NK, &inFile, &angle_file );
      }
      else
      {
        // get the maximum Legendre order
        Info("LoadGammaData","MF=14 Legendre expansions for gammas");
        int max_order = angle_file.get_order( mf14_NK, mf14_NI );
        angle_file.close();
        Info("LoadData","Closing file "+endf_file_name(14,ENDL.T)+"...");
        angle_file.open( 14, ENDL.T );
        MF14_list e_dist( mf14_NI, mf14_NK, max_order, &inFile, &angle_file );
        e_dist.master( mf12_NK );
      }
    }
    
    // anisotropic gammas
    else
    {
    
      // there is just one line, and it is anisotropic
      if( ( mf14_NK == 1 ) && ( mf14_NI == 0 ) )
      {
        Info("LoadGammaData","Doing MF=4 table for 1 line");
        mf4_table Tabular;    //Create the tabulated object
        Tabular.master( angle_file );
      }
      
      // there are multiple, and they are anisotropic
      else
      {
        Info("LoadGammaData","Doing MF=14 table for multiple lines");
        MF14_table Tabular;    //Create the tabulated object
        Tabular.master( inFile, angle_file, mf12_NK, mf14_NK ); 
      }
    }
  }
  
  //We have transition probability arrays
  else if ( LO == 2 ) 
  {
    if( mf14_LI == 1 )
    {
      Info("LoadGammaData","Doing MF=12 deexcitation");
      MF12_deexcite e_dist; 
      e_dist.master( AWR, LG, inFile );
    }
    else
    {
      Unimplemented("LoadGammaData","Implement anisotropic gammas for deexcitation");
    }
  }
  else // something horribly wrong
  {
    SevereError("LoadGammaData",
        pastenum("There is no LO = ",LO)+" option for MF=12 data!!");
  }

  
}

// --------------- LoadMF13Data ------------------------
void LoadMF13Data( mf13_file& inFile ) 
{
  int ZA;
  double AWR;
  int NK;

  // read the first line
  inFile.read_line1(
    &ZA,             // target Z & A 
    &AWR,            // target mass
    &NK              // number of discreet photons (inc continuum)
  );
   
  // Reset the header information, overwrite the previous reaction
  ENDL.set_x0( 0.0 );      // the Q value
  ENDL.set_I_number( 0 );  // recompose the header
  if ( !ENDL.write_file )
  {
    Info("LoadMF13Data",pastenum("Skipping mf: ",ENDL.F)+pastenum(" mt: ",ENDL.T));
    return;
  }
  if( ENDL.T == 3 )
  {
    Info("LoadMF13Data","Doing mf13 table for mt3");
    C55.MT3_gammas.master(NK, inFile);
    C55.has_MT3 = true;
  }
  else if( ENDL.T == 4 )
  {
    Info("LoadMF13Data","Doing mf13 table for mt4");
    C55.MT4_gammas.master(NK, inFile);
    C55.has_MT4 = true;
  }
  else
  {
    Warning("LoadMF13Data","Write new code for gamma multiplicity.");
  }
}
