/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1869 $
 * $Date: 2006-05-15 12:56:46 -0700 (Mon, 15 May 2006) $
 * $Author: dbrown $
 * $Id: logger.cpp 1869 2006-05-15 19:56:46Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

#include <fstream>
#include <string>
#include <sstream>
#include "version.hpp"
#include "logger.hpp"
#include "convert.hpp"
#include "global_params.hpp"

extern GlobalParameterClass Global;

//! output for MFMT
ostream& operator<<(ostream& out, const MFMT& mfmt){
    out << "MF: "<<mfmt.MF<<" MT: "<<mfmt.MT;
    if (mfmt.C!=-1 && mfmt.I!=-1 && mfmt.S!=-1) {
        out << " c: "<<mfmt.C<<" i: "<<mfmt.I<<" s: "<<mfmt.S;
    }
    return out;
}

//! output for logEntry
ostream& operator<<(ostream& out, const logEntry& ent){
    switch (ent.severity) {
        case 0: 
            out << rjust("Info",14); 
            break; 
        case 1: 
            out << rjust("Unimplemented",14);  
            break; 
        case 2: 
            out << rjust("Warning",14); 
            break; 
        case 3: 
            out << rjust("Severe Error",14);
            break; 
        case 4: 
            out << rjust("Fatal Error",14); 
    }
    out <<", "<< ent.message;
    return out;
}

void MessageLogger::logMessage(int MF, int MT, string msg, int severity){
    MFMT thisMFMT(MF=MF,MT=MT);
    while (find(thisMFMT)!=end()){
        thisMFMT.index+=1;
    }
    insert(make_pair(thisMFMT,logEntry(msg,severity)));
}
    
void MessageLogger::logMessage(int MF, int MT, int C, int I, int S, string msg, int severity){
    MFMT thisMFMT(MF,MT,C,I,S);
    while (find(thisMFMT)!=end()){
        thisMFMT.index+=1;
    }
    insert(make_pair(thisMFMT,logEntry(msg,severity)));
}

//! main output routine for the MessageLogger
bool MessageLogger::write(void){
    bool isEndfDoc(false);
    ifstream inStream(inLogFile.c_str());
    stringstream oldStuff("");
    char buffer[81];
    if (inStream.good()) {
        while (inStream.getline(buffer,sizeof(buffer))){oldStuff<<buffer;oldStuff<<"\n";}
        isEndfDoc=true;
    } else {
        cout << "Info: Could not read "<<inLogFile<<", will not include ENDF documentation in log file"<<endl;
    }
    ofstream outStream(outLogFile.c_str());
    if (!outStream.good()) return false;
    if (isEndfDoc) {
        outStream << center(60*string("="),62) << endl;
        outStream << center("ENDF File Documentation",62) << endl;
        outStream << center(60*string("="),62) << endl;
        outStream << oldStuff.str() << endl << endl;
    }
    if (Global.Value( "skip_logging" )>0.0) return true;
    //Pretty banner to set off start of program & print the version information
    outStream << 11*string(" ") << "+" << 40*string("-") << "+"<<endl;
    outStream << 11*string(" ") << "|" << center("fete: from ENDF to endl...",40)<< "|"<<endl;
    outStream << 11*string(" ") << "|" << center("Version "+string(FETE_VERSION),40) << "|" <<endl;
    outStream << 11*string(" ") << "|" << center("Revision "+string(FETE_REVISION),40) << "|" <<endl;
    outStream << 11*string(" ") << "|" << center("Last SVN commit "+string(FETE_VERSION_DATE),40) << "|" <<endl;
    outStream << 11*string(" ") << "+" << 40*string("-") << "+"<<endl;
    outStream << endl;
    if (this->size()>0) {
        outStream << center(60*string("="),62) << endl;
        outStream << center("Problems encountered during translation",62) << endl;
        outStream << center(60*string("="),62) << endl;
        for (MessageLogger::const_iterator it=begin(); it!=end(); ++it){
            stringstream buf;
            buf<<it->first;
            outStream << ljust(buf.str(),35) << it->second << endl;
        }
    } else {
        outStream << center(60*string("="),62) << endl;
        outStream << center("No problems encountered during translation!!",62) << endl;
        outStream << center(60*string("="),62) << endl;
    }
    return true;
}
