/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1869 $
 * $Date: 2006-05-15 12:56:46 -0700 (Mon, 15 May 2006) $
 * $Author: dbrown $
 * $Id: logger.hpp 1869 2006-05-15 19:56:46Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

#ifndef __LOGGER_HPP
#define __LOGGER_HPP

#include <map>
#include <string>
#include <iostream>
#include "version.hpp"
#include "global_params.hpp"

using namespace std;

//! MF & MT of of the file that caused the problem.  Use this as a key in a map.
class MFMT{
public:
    int MF,MT;
    int index;
    int C,I,S; // not really part of key, but useful to keep the C,I,S near the MF,MT, since no one remembers mapping
    MFMT(int mf, int mt, int ind=0, int c=-1, int i=-1, int s=-1): MF(mf), MT(mt), index(ind), C(c), I(i), S(s){}
    bool operator<(MFMT other) const{
        if (other.MT!=MT) {return other.MT>MT;} 
        else if (other.MF!=MF) {return other.MF>MF;} 
        else {return other.index>index;}
    }
    friend ostream& operator<<(ostream& out, const MFMT& mfmt);
};

//! Severity of the problem and the message that goes with it.  Use this as the value in the map.
class logEntry{
public:
    string message;
    int severity;
    logEntry(string msg, int sev): message(msg), severity(sev){}
    friend ostream& operator<<(ostream& out, const logEntry& ent);
};

//! Map to hold the list of problems.  We append this list to the documentation of the evaluation.
class MessageLogger : public map< MFMT, logEntry > {
public:
    string inLogFile;
    string outLogFile;
    MessageLogger(string infile="mf01mt451", string outfile="documentation.txt"): 
        inLogFile(infile), outLogFile(outfile){}
    ~MessageLogger(void){}
    bool write(void);
    void logMessage(int MF, int MT, string msg, int severity);
    void logMessage(int MF, int MT, int C, int I, int S, string msg, int severity);
};

#endif
