/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: mappings.hpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// header for the mappings class

#ifndef DEF_MAPPINGS
#define DEF_MAPPINGS

#include <cmath>

#include "nuclei.hpp"

// ----------- Pair class for E-mu structure --------------
// energy and cosine
//! Stores energy and direction cosine pair used in reference frame mapping
class Pair
{
public:
  double E;       //  energy of ejected particle
  double mu;      //  direction cosine

  //! Default constructor
  Pair();
  //! Specific pair constructor
  Pair(double energy, double cosine);
  //! Default destructor
  ~Pair();
};

//! Returns A - B
Pair operator-(const Pair& A, const Pair& B);

//! Returns area of the triangle generated by A, B, and O
double Pair_area(const Pair& A, const Pair& B);

// ----------- class mappings -----------------
//! Class that maps energy-angle pairs between reference frames
class mappings
{
private:
  // pointers to the particles
  Target *_target;   
  Projectile *_projectile;
  Product *_ejectum;   

  double _E_in;  // energy of the incident particle

public:
  double E_transl;     // translational energy from initial collision

  //! Default constructor
  mappings( );

  //! Constructor with particle data
  mappings(Target *target, Projectile *projectile,
    Product *ejectum, double E_in);

  //! Default destructor
  ~mappings();

  //! Set the particle data
  void set_map(Target *target, Projectile *projectile,
    Product *ejectum, double E_in);

  //! Maps from lab frame to center-of-mass
  Pair lab_to_cm(Pair& lab);

  //! Maps from center-of-mass to lab
  Pair cm_to_lab(Pair& cm);

  //! Jacobian of the mapping
  double J_cm_to_lab( double E_cm, double E_lab );

  //! Returns the minimal cosine (lab frame) for this exit energy
  double get_min_mu(double E_cm);

  //! Calculates the lab frame exit energy extremes
  void get_E_range(double E_cm, double mu, double *E_back,
    double *E_ahead);
  // Return the lab frame exit energies for given center-of-mass exit
  // energy and lab angle mu.
  // If there are 2 such energies, the smaller one is E_back.
  // If there is only 1, it is E_ahead and E_back = -1.
  // If there are none, then both E_ahead and E_back are set to -1.

  //! Calculates the lab energy for given c-m energy and lab mu
  double get_lab_E( double E_cm, double mu_lab, int signum );
  // signum designates the sign of the square root.
};

#endif

