/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: messaging.cpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

#include <string>
#include <iostream>
#include <sstream>
#include "global_params.hpp"
#include "messaging.hpp"
#include "endl_formats.hpp"
#include "logger.hpp"

extern GlobalParameterClass Global;
extern ENDLClass ENDL;
extern MessageLogger messageLog;

using namespace std;

//! Always write out a Fatal Error message, then exit badly
void FatalError(string routine, string error, bool writeLog){
    string msg = "fete "+routine+": "+error;
    cerr << "Fatal Error "<<msg<<endl;
    messageLog.logMessage(ENDL.F, ENDL.T, ENDL.C, ENDL.I, ENDL.S, msg, 4);
    if (writeLog) messageLog.write();
    exit(-1);
};

//! Always write out a Severe Error message, then attempt to throw an exception
void SevereError(string routine, string error){
    string msg = routine+": "+error;
    messageLog.logMessage(ENDL.F, ENDL.T, ENDL.C, ENDL.I, ENDL.S, msg, 3);
    throw(string("Severe Error "+msg));
};

//! Only show a warning if required
void Warning(string routine, string warning){
    if (MESSAGELEVEL_WARN) {
        string msg = "fete "+routine+": "+warning;
        cerr << "Warning "<<msg<<endl;
        messageLog.logMessage(ENDL.F, ENDL.T, ENDL.C, ENDL.I, ENDL.S, msg, 2);
    }
};

//! Only show information if required
void Info(string routine, string info){
    if (MESSAGELEVEL_INFO) {
        string msg = "fete "+routine+": "+info;
        cout << "Info "<<msg<<endl;
        //messageLog.logMessage(ENDL.F, ENDL.T, msg, 0); // never log Info statements
    }
};

//! Only show information if required
void Unimplemented(string routine, string info){
    if (MESSAGELEVEL_INFO) {
        string msg = "fete "+routine+": "+info;
        cout << "Unimplemented "<<msg<<endl;
        messageLog.logMessage(ENDL.F, ENDL.T, ENDL.C, ENDL.I, ENDL.S, msg, 1);
    }
};

//! gizmo to paste doubles on the backs of strings
string pastenum(string front, double back){
    ostringstream tmp; 
    tmp << front << back; 
    return tmp.str();
};

//! gizmo to paste ints on the backs of strings
string pastenum(string front, int back){
    ostringstream tmp; 
    tmp << front << back; 
    return tmp.str();
};
