/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: mf13classes.hpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// header for the MF=13 translation code

#ifndef MF13CLASSES
#define MF13CLASSES

#include "list_2d.hpp"
#include "ENDF_file.hpp"
#include "mf15classes.hpp"
#include "multiplicity.hpp"

using namespace std;

// ----------- class MF13_list -----------------
// derive this class from two_d_list
//! Class to handle the photon production cross sections
class MF13_list : public two_d_list< multiplicity >
{
public:
  //! continuum energy distributions
  MF15_list cont_data;

  //! continuum multiplicity
  multiplicity cont_multiple;

  //! do we have discrete gammas (for s = 3)
  bool with_discrete;

  //! Handles an MF=13 file
  void master( int NK, mf13_file& inFile );

  //! Routine to read in ENDF/B-VI probabilities of discrete gamma production
  void read_data( int NK, mf13_file& inFile );

  //! Writes a c = 55, i = 0, s = 3 ENDL file
  void write_endl();

  //! Writes a c = 55, i = 1, s = 3 ENDL file
  void isotropic();

  //! Reads gamma production and energy distributions for the continuum
  void continuum( int NR, int NP, mf13_file& inFile );

  //! Writes C55 data for the continuum
  void write_continuum( );

};

// ----------- class C55_gammas -----------------
// derive this class from two_d_list
//! Class to handle all of the photon production cross sections
class C55_gammas
{
public:

  MF13_list MT3_gammas;
  MF13_list MT4_gammas;

  bool has_MT3;
  bool has_MT4;

  C55_gammas( )
    {
      has_MT3 = false;
      has_MT4 = false;
    }

  ~C55_gammas( )
    {}

  //! Sets the weights of the energy distributions according to the photon production cross section.
  void set_weights( );

  //! Writes the c55 files
  void write_endl();
};

#endif
