/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1748 $
 * $Date: 2006-03-14 12:18:15 -0800 (Tue, 14 Mar 2006) $
 * $Author: dbrown $
 * $Id: mf14table.hpp 1748 2006-03-14 20:18:15Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// header for the translation of MF=14 tabular data

#ifndef MF14TABLE
#define MF14TABLE

#include "mf4classes.hpp"
#include "mf12classes.hpp"
#include "mf6classes.hpp"

using namespace std;

// ----------- class MF14_table_raw -----------------
//! This class reads and stores the MF=14 tabular data in ENDF format.
//! The data is a list of mf4_table, one for each gamma.
class MF14_table_raw : public list< mf4_table >
{
private:
  void fill_mu( );

public:
  // ! Read the ENDF data
  void read_data( mf14_file& inFile, int num_gamma );

  //! Ensure common E_in values
  void fill_E_in( );

};

// ----------- class MF14_table -----------------
//! This class is for anisotropic gamma tables (LTT = 2) which are not c55 data
class MF14_table : public ENDL_table
{
private:
  // the total multiplicity
  multiplicity multiple;

  // The mf12 multiplicity data for the lines
  MF12_raw line_mult;

  // The ENDF mf14 tabular data
  MF14_table_raw endf_mf14;

  //! Reads the gamma multiplicities
  void read_mf12( mf12_file& mult_file, int num_mf12 );

  //! Weights the raw mf14 data by their multiplicity
  void mult_weight( );

  //! Puts the data into ENDL order
  void ENDL_order( );

  //! Converts the lines to delta-functions
  void widen_deltas( );

  //! Creates the ENDL i = 1 angular distributions
  void make_i1( );

public:
  //! Handles the mf12 and mf14 tabular data
  void master( mf12_file& mult_file, mf14_file& angle_file,
    int num_mf12, int num_mf14 );
};

#endif


