/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: mf4classes.hpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

//header for the classes used in translating the MF=4 files

#ifndef MF4CLASSES
#define MF4CLASSES

#include "list_2d.hpp"
#include "ENDF_file.hpp"

using namespace std;

// ----------- class two_d_isotropic -----------------
// derive this class from two_d_list
//! Class to handle isotropic distributions
class two_d_isotropic : public two_d_list<one_d_isotropic>
{
public:
  //! Makes an isotropic list
  void expand_data(int MT);

};

// ----------- class two_d_Legendre -----------------
// derive this class from two_d_list
//! Class to handle Legendre data
class two_d_Legendre : public two_d_list<one_d_Legendre>
{
public:
  //! Reads in all the data and expands into linearly interpolable pointwise data
  void expand_data(mf4_file& inFile, int NE, int MF);

  //! Reads in data for one incident neutron energy and expands
  //! *done = true if the incident energy is at or above the maximum
  void one_E_in( mf4_file& inFile, int MF, bool *done );

  //! Converts data so that it is linearly interpolable in incident energy
  void expand_interp();
};


// ----------- class mf4_table -----------------
//! Class that handles tabular data
class mf4_table : public two_d_table
{
public:
  //! Manages everything
  void master( mf4_file& inFile );

  //! Reads in all the data
  void read_data( mf4_file& inFile, int NE );

  //! Reads in data for a particular incident energy
  void one_E_in( mf4_file& inFile );

};

// ----------- class mixed_mf4 -----------------
//! Class that handles Legendre data at low incident energies and tables at high energies
class mixed_mf4 : public two_d_Legendre
{
private:
  // For the tabular data
  mf4_table table_data;

public:
  // Manage everything
  void master( mf4_file& inFile );
};

// ----------- class gen_mf4 -----------------
// derive this class from two_d_table
//! Class to handle data coming from MF=6 files
class gen_mf4 : public two_d_table
{
// This class is used by mf6 files, and it may
// contain Legendre data for some incident energies
// and tabular data for others.  We convert it to a
// 2-d table.
public:
  //! Interprets the data
  void master( mf6_file& inFile );

  //! Interprets Legendre data for 1 incident energy
  void one_legendre( mf6_file& inFile, double e_in, int NL );

  //! Interprets tabular data for 1 incident energy
  void one_table( mf6_file& inFile, double e_in, int NL );

};

#endif

