/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: mf6_gammas.cpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// implementation of the class mf6_gammas

#include "mf6_gammas.hpp"

extern ENDLClass ENDL;
extern GlobalParameterClass Global;

// ********* for class mf6_table *************
// ----------- mf6_table::new_one_d -----------------
mf6_table::iterator mf6_table::new_one_d( mf6_table::iterator where )
{
  SevereError("mf6_table::new_one_d", "Implement the routine");
  return 0;
}

// ----------- mf6_table::read_data -----------------
void mf6_table::read_data( mf6_file& inFile, int LEP, int NE )
// read in the ENDF/B-VI tabulated  data
// LEP is the interpolation type
// NE is the number of incident energies
{
  // handle each incident neutron energy
  for ( int iNE=0; iNE < NE; iNE++ )
  {
    one_E_in( inFile, LEP );
  }
}

// ------------ mf6_table::one_E_in ---------
void mf6_table::one_E_in( mf6_file& inFile, int LEP )
{
  double E_1;   // incident neutron energy
  int ND;   // number of discrete energies
  int NEP;  // number of secondary energies

  string linebuff;
  string strbuff;
  
  inFile.gamma_ein( &E_1,   // incident energy
                    &ND,    // number of discrete energies
                    &NEP ); // number of E_out values
  
  // change to MeV
  E_1 *= ENDL.eV2MeV;
  // this E_1 corresponds to a new link in the 2d list
  one_d_table e_in_link;
  e_in_link.E_in() = E_1;
  insert(end(), e_in_link);
  mf6_table::iterator this_ein = end();
  --this_ein;

  // read the data for this E_in
  // we may need a list of gamma lines
  one_d_table gamma_lines;
  gamma_lines.E_in() = E_1;

  for( int iNP=0; iNP < NEP; iNP++ ) //Loop over NEP data pairs
  { 
    int iP=iNP % 3;  //There are 3 pairs per line
    if ( iP == 0 )
    {
      getline( inFile, linebuff ); //Read in new line
    }
    strbuff = linebuff.substr( iP*22, 22 );  // grab the right pair
    double X, Y; //Real variables we will put the data pair
    read_dd(&strbuff, &X, &Y );
    // to MeV
    X *= ENDL.eV2MeV;
    dd_link XYdata( X, Y );
    // is this a line?
    if( iNP < ND )
    { 
      if ( Y > 0.0 )
      {
        // find where to insert the line
        one_d_table::iterator next_line = gamma_lines.find_next( X );
        gamma_lines.insert( next_line, XYdata );
      }
    }
    else
    {
      this_ein->insert( this_ein->end( ), XYdata );
    }
  }
  // expand the interpolation
  this_ein->dd_list::expand_interp( LEP, ENDL.Max_E_out );

  if( gamma_lines.size( ) > 0 )
  {
    gamma_lines.widen_delta( );
    if( this_ein->size( ) > 0 )
    {
      // add the lines to the continuum
      (*this_ein) += gamma_lines;
    }
    else
    {
      // just copy the lines
      this_ein->copy( gamma_lines );
    }
  }
  this_ein->renorm( );
}

// ********* for class mf6_gamma *************
// ----------- mf6_gamma::read_data -----------------
void mf6_gammas::read_data( mf6_file& inFile, int LEP,
  int NR, int num_E_in )
// read in the gamma data and make it into a three_d_list
{
  // get the interpolation rules
  inFile.get_regions( NR, NBT, INT );

  // make the isotropic energy distributions
  mf6_table e_dist;
  e_dist.read_data( inFile, LEP, num_E_in );

  // now make a three_d_link for each incident energy
  for(mf6_table::iterator e_dist_ptr = e_dist.begin();
      e_dist_ptr != e_dist.end(); ++e_dist_ptr)
  {
    mf6_table e_in_link;
    insert(end(), e_in_link);

    // make a pointer to the new 3-d link
    mf6_gammas::iterator this_link = end();
    --this_link;

    // Expand this data into a 3-d link
    expand_E_in(e_dist_ptr, this_link);

    // add a link to the cosine list
    one_d_table new_cos;
    cosines.insert(cosines.end(), new_cos);

    // expand this cosine link
    two_d_table::iterator cos_ptr = cosines.end();
    --cos_ptr;
    cos_ptr->E_in() = e_dist_ptr->E_in();

    // make 2 isotropic links
    dd_link new_link(-1.0, 0.5);
    cos_ptr->insert(cos_ptr->end(), new_link);
    new_link.x = 1.0;
    cos_ptr->insert(cos_ptr->end(), new_link);
  }
  // print the ENDL i=1 and i=3 files
  write_endl( );
}
// ----------- mf6_gammas::expand_E_in -----------------
void mf6_gammas::expand_E_in(mf6_table::iterator e_dist_ptr,
  mf6_gammas::iterator this_link)
// make isotropic double differential data for 1 incident energy
{
  this_link->E_in() = e_dist_ptr->E_in();

  for (int count = 0; count < 2; ++count)
  {
    double this_mu = -1.0 + 2*count;
    // mu corresponds to a new 2d link in the list for this E_in
    one_d_table mu_link;
    mu_link.mu() = this_mu;
    this_link->insert(this_link->end(), mu_link);

    // add energy links for this mu
    mf6_table::iterator mu_ptr = this_link->end();
    --mu_ptr;
    // copy the entries from e_dist_ptr
    for(dd_list::iterator e_dist_link = e_dist_ptr->begin();
        e_dist_link != e_dist_ptr->end(); ++e_dist_link)
    {
      dd_link e_out_link(e_dist_link->x, e_dist_link->y);
      mu_ptr->insert(mu_ptr->end(), e_out_link);
    }
  }
}
