/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: mf6_gammas.hpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// header for the special class for mf6 gammas

#ifndef MF6_GAMMAS_DEF
#define MF6_GAMMAS_DEF

#include "list_3d.hpp"


// ----------- class mf6_table -----------------
//! Class to handle the MF=6 tabular data
class mf6_table : public three_d_link< one_d_table >
{
public:
  // ********* implement virtual functions *******************
  // Append a new mf6_table
  mf6_table::iterator new_one_d( mf6_table::iterator where );
  // ********** end of virtual functions *********************

  //! Reads in all the data from inFile for NE energies
  void read_data( mf6_file& inFile, int LEP, int NE );

  //! Reads in the data for a particular energy
  void one_E_in( mf6_file& inFile, int LEP );

};

// ----------- class mf6_gammas ---------------------
// derive this class from three_d_list
//! Class to handle the gammas from MF=6 files.
class mf6_gammas : public three_d_list< mf6_table >
{
public:
  //! Reads in the gamma data and makes a three_d_list
  void read_data( mf6_file& inFile, int LEP, int NR, int num_E_in );

  //! Creates isotropic double differential data for a single incident neutron energy
  void expand_E_in(mf6_table::iterator e_dist_ptr,
    mf6_gammas::iterator this_link);

};

#endif
