/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1866 $
 * $Date: 2006-05-15 09:12:12 -0700 (Mon, 15 May 2006) $
 * $Author: dbrown $
 * $Id: multiplicity.hpp 1866 2006-05-15 16:12:12Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// header for the multiplicity class

#ifndef MULTIPLICITY_CLASS
#define MULTIPLICITY_CLASS

#include "distrib_1d.hpp"
#include "ENDF_file.hpp"

// *********** class multiplicity *************
//! Class to handle ENDF multiplicity data.
class multiplicity : public one_d_table
{
public:  
  // to enable the use of the sum_lists routine
  double weight;

  //! Default constructor
  multiplicity():weight(0.0){}
  
  //! Default destructor
  ~multiplicity(){}

  //! Label for this list for gamma multiplicities
  inline double& E_gamma()
  {
    return tag;
  }

  //! Read in and expands the multiplicity 2-column data
  void read_data( int NP, ENDF_file& inFile);

  //! Checks the multiplicity to make sure it is constant, as it should be
  bool check_const_mult( );
  bool check_const_mult( double first_mult );
  
  //! Sets a floor/ceiling value on the multiplicity for those times where 
  //! it is not constant, but a minimum value is requires
  void set_min_mult( double min_mult );
  void set_max_mult( double max_mult );

  //! Returns the constant multiplicity as may be used in an (n, Xn) reaction
  int get_mult( );

  //! Sets the multiplicity to a constant, if we know it from another method (say the reaction MT)
  void set_mult( int mult );

  //! Scales photon multiplicity by the cross section to get photon production
  void scale_by_xs();

  //! Divides photon production by the cross section to get photon multiplicity
  void divide_by_xs();

  //! Needed to combine lists of discrete gamma multiplicities
  void list_interp( double e_in, multiplicity& left_list,
    multiplicity& right_list );

  //! Used to verify that the multiplicity of a single line doesn't exceed 1
  void check_count( ); 

private:

protected:

};

#endif
