/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1735 $
 * $Date: 2006-02-09 13:47:26 -0800 (Thu, 09 Feb 2006) $
 * $Author: dbrown $
 * $Id: nuclei.cpp 1735 2006-02-09 21:47:26Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// inplementation of the Particle class

#include "nuclei.hpp"
#include "bdfls_tools.hpp"
#include "messaging.hpp"

extern bdflsClass bdfls;

// *************** Nuclei class *******************
// ----------- Nuclei::Nuclei -----------------
Nuclei::Nuclei()
{
  ZA = 0;
  Z = 0;
  N = 0;
  A = 0;
  AWR = 0.0;
  tau = 0.0;
}
// ----------- Nuclei::Nuclei -----------------
Nuclei::Nuclei(int za)
{
  ZA = za;
  crack_ZA();

  // also set the atomic weight
  if ( za == 0 )
  {
    AWR = 0.0;
  }
  else
  {
    mass_life_list::iterator XYptr;
    if(!bdfls.Mass_Life.at(za, XYptr))
    {
      SevereError("Nuclei::Nuclei",pastenum("no mass found for za: ",za));
    }
    AWR = XYptr->Mass();
  }
}
// ----------- Nuclei::~Nuclei -----------------
Nuclei::~Nuclei()
{
}
// ----------- Nuclei::set -----------------
void Nuclei::set(int za)
{
  ZA = za;
  crack_ZA();

  // also set the atomic weight
  if ( za == 0 )
  {
    AWR = 0.0;
  }
  else
  {
    mass_life_list::iterator XYptr;
    if(!bdfls.Mass_Life.at(za, XYptr))
    {
      SevereError("Nuclei::set",pastenum("no mass found for za: ",za));
    }
    AWR = XYptr->Mass();
  }
}
// ----------- Nuclei::crack_ZA -----------------
void Nuclei::crack_ZA()
// get the proton (Z), neutron (N), and mass (A) numbers for the ZA
{
  Z = ZA/1000;
  A = ZA - 1000*Z;
  if(A == 0)
  {
    N = 0;  // a natural target
  }
  else
  {
    N = A - Z;
  }
}
