/*
 * ******** fete: From ENDF To ENDL *********
 * 
 * Copyright (c) 2006, The Regents of the University of California. 
 * All rights reserved.
 * 
 * Produced at the Lawrence Livermore National Laboratory. 
 * Written by David A. Brown, Gerry Hedstrom, Tony Hill
 * 
 * This file is part of fete v1.0  (UCRL-CODE-218718)
 * 
 * Please read the COPYING file for "Our Notice and GNU General 
 * Public License" in the root of this software distribution.  
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License (as published by 
 * the Free Software Foundation) version 2, dated June 1991. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms 
 * and conditions of the GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * $Revision: 1748 $
 * $Date: 2006-03-14 12:18:15 -0800 (Tue, 14 Mar 2006) $
 * $Author: dbrown $
 * $Id: record_types.cpp 1748 2006-03-14 20:18:15Z dbrown $
 * 
 * ******** fete: From ENDF To ENDL *********
 */

// Function methods for the different record types

#include "record_types.hpp"
#include "convert.hpp"
#include "messaging.hpp"

string toupper(const string& s)
{
	string t = s;
	int length = t.length();
	for (int i = 0; i < length; i++) t[i] = toupper(t[i]);
	return t;
}

void read_ddiiii(string *pS, 
		 double *pR1, double *pR2, 
		 int *pI1, int *pI2, int *pI3, int *pI4)
{
  *pR1 = stod( pS->substr(  0, 11 ) );
  *pR2 = stod( pS->substr( 11, 11 ) );
  *pI1 = stoi( pS->substr( 22, 11 ) );
  *pI2 = stoi( pS->substr( 33, 11 ) );
  *pI3 = stoi( pS->substr( 44, 11 ) );
  *pI4 = stoi( pS->substr( 55, 11 ) );
}

void read_idiiii(string *pS, 
		 int *pI1, 
		 double *pR1, 
		 int *pI2, int *pI3, int *pI4, int *pI5)
{
  *pI1 = stoi( pS->substr(  0, 11 ) );
  *pR1 = stod( pS->substr( 11, 11 ) );
  *pI2 = stoi( pS->substr( 22, 11 ) );
  *pI3 = stoi( pS->substr( 33, 11 ) );
  *pI4 = stoi( pS->substr( 44, 11 ) );
  *pI5 = stoi( pS->substr( 55, 11 ) );
}

void read_iiidiiddd(string *pS, 
		    int *pI1, int *pI2, int *pI3,
		    double *pR1, 
		    int *pI4, int *pI5, 
		    double *pR2, double *pR3, double *pR4 )
{
  *pI1 = stoi( pS->substr(  0,  6 ) );
  *pI2 = stoi( pS->substr(  6,  3 ) );
  *pI3 = stoi( pS->substr(  9,  3 ) );
  *pR1 = stod( pS->substr( 12, 12 ) );
  *pI4 = stoi( pS->substr( 24,  7 ) );
  *pI5 = stoi( pS->substr( 31,  3 ) );
  *pR2 = stod( pS->substr( 34, 12 ) );
  *pR3 = stod( pS->substr( 46, 12 ) );
  *pR4 = stod( pS->substr( 58, 12 ) );
}

void read_iiiddddd(string *pS, 
		    int *pI1, int *pI2, int *pI3, 
		    double *pR1, double *pR2,
		    double *pR3, double *pR4, double *pR5 )
{
  *pI1 = stoi( pS->substr(  0,  2 ) );
  *pI2 = stoi( pS->substr(  2,  3 ) );
  *pI3 = stoi( pS->substr(  5,  3 ) );
  *pR1 = stod( pS->substr(  8, 12 ) );
  *pR2 = stod( pS->substr( 20, 12 ) );
  *pR3 = stod( pS->substr( 32, 12 ) );
  *pR4 = stod( pS->substr( 44, 12 ) );
  *pR5 = stod( pS->substr( 56, 12 ) );
}

void read_d(string *pS, 
	    double *pR )
{
  *pR = stod( pS->substr(  0, 11 ) );
}

void read_dd(string *pS, 
	     double *pR1, double *pR2 )
{
  *pR1 = stod( pS->substr(  0, 11 ) );
  *pR2 = stod( pS->substr( 11, 11 ) );
}

void read_ddd(string *pS, 
	     double *pR1, double *pR2, double *pR3 )
{
  *pR1 = stod( pS->substr(  0, 11 ) );
  *pR2 = stod( pS->substr( 11, 11 ) );
  *pR3 = stod( pS->substr( 22, 11 ) );
}

void read_dddddd(string *pS, 
		 double *pR1, double *pR2, double *pR3,
                 double *pR4, double *pR5, double *pR6 )
{
  *pR1 = stod( pS->substr(  0, 11 ) );
  *pR2 = stod( pS->substr( 11, 11 ) );
  *pR3 = stod( pS->substr( 22, 11 ) );
  *pR4 = stod( pS->substr( 33, 11 ) );
  *pR5 = stod( pS->substr( 44, 11 ) );
  *pR6 = stod( pS->substr( 55, 11 ) );
}

void read_ii(string *pS, 
	     int *pI1, int *pI2 )
{
  *pI1 = stoi( pS->substr(  0, 11 ) );
  *pI2 = stoi( pS->substr( 11, 11 ) );
}

void read_date(string *pS, 
	     int *pI1 )
{
  string smonth;
  int year,month,day;

  //get the month and year of the evaluation
  smonth = toupper(pS->substr( 27,3 ));
  year = stoi( pS->substr(  30, 2 ) );

  //Convert the character month to a string
  if( smonth == "JAN" ) 
    {
      month = 1;
    }
  else if( smonth == "FEB" ) 
    {
      month = 2;
    }
  else if( smonth == "MAR" ) 
    {
      month = 3;
    }
  else if( smonth == "APR" ) 
    {
      month = 4;
    }
  else if( smonth == "MAY" ) 
    {
      month = 5;
    }
  else if( smonth == "JUN" ) 
    {
      month = 6;
    }
  else if( smonth == "JUL" ) 
    {
      month = 7;
    }
  else if( smonth == "AUG" ) 
    {
      month = 8;
    }
  else if( smonth == "SEP" ) 
    {
      month = 9;
    }
  else if( smonth == "OCT" ) 
    {
      month = 10;
    }
  else if( smonth == "NOV" ) 
    {
      month = 11;
    }
  else if( smonth == "DEC" ) 
    {
      month = 12;
    }
  else
    {
      SevereError("read_date",smonth+" is not a recognized month!");
    }
  
  //Since there is no "day" associated with the evaulation date, we use "01"
  day=1;

  //ENDL date format is used    
  *pI1 = year*10000+month*100+day;
}
// *************** read_yi **********************
int read_yi( string *pS )
{
  int yi;

  string upS = toupper(*pS);

  if (upS.find("NEUTR")!=string::npos) yi = 1;

  else if (upS.find("PROTO")!=string::npos) yi = 2;

  else if (upS.find("DEUTE")!=string::npos) yi = 3;

  else if (upS.find("TRITO")!=string::npos) yi = 4;

  else if (upS.find("HELIU")!=string::npos) yi = 5;

  else if (upS.find("ALPHA")!=string::npos) yi = 6;

  else if ( (upS.find("PHOTO")!=string::npos) || (upS.find("GAMMA")!=string::npos) ) yi = 7;

  else
  {
    SevereError( "read_yi","Cannot identify incident particle" );
  }

  return yi;
}
